import os
import shutil

import pytest


@pytest.fixture(autouse=True, scope="session")
def tmp_dir(tmpdir_factory):
    directory = tmpdir_factory.mktemp('test')
    os.chdir(str(directory))
    yield directory
    shutil.rmtree(str(directory), ignore_errors=True)


@pytest.fixture
def image_file(tmp_dir):
    """Pass a JPEG file resource as an argument to the unit tests."""
    file = os.path.join(os.path.dirname(__file__), 'test.jpg')
    test_file = str(tmp_dir.join('test.jpg'))
    shutil.copyfile(file, test_file)

    return test_file



import os

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QFileDialog
import pytest

from mausoleum import application


@pytest.fixture
def window(qtbot):
    """Create an instance of Mausoleum to pass to test functions.

    window is used as a pytest fixture that allows passing it to test functions. Qtbot
    uses qApp to open a new main window for each test function.
    """
    new_window = application.Mausoleum()
    qtbot.add_widget(new_window)
    new_window.show()
    yield new_window


@pytest.fixture
def name():
    """Use test1.tomb as the tomb name to pass to test functions."""
    return 'test1.tomb'


@pytest.fixture
def key():
    """Use test1.tomb.key as the tomb key name to pass to test functions."""
    return 'test1.tomb.key'


@pytest.fixture
def password():
    """Use test_password as the tomb password to pass to test functions."""
    return 'test_password'


def test_window_title(window):
    """Check the window title of the main window."""
    assert window.windowTitle() == 'Mausoleum'


def test_create_page_failure(window, qtbot):
    """Test tomb creation with key passwords that don't match."""
    button = window.create_page.create_button
    window.create_page.key_password.setText('1')
    window.create_page.confirm_password.setText('2')
    qtbot.mouseClick(button, Qt.LeftButton)
    assert window.create_page.message.text() == 'Key Passwords Do Not Match'


def test_create_page(window, qtbot, name, key, password):
    """Test tomb creation and tomb opening."""
    window.create_page.tomb_name.setText(name)
    window.create_page.key_name.setText(key)
    window.create_page.key_password.setText(password)
    window.create_page.confirm_password.setText(password)
    window.create_page.random_checkbox.setChecked(True)
    window.create_page.open_checkbox.setChecked(True)
    window.create_page.debug = True
    button = window.create_page.create_button
    qtbot.mouseClick(button, Qt.LeftButton)
    assert window.create_page.message.text() == 'Tomb Opened Successfully'


def test_close_page_close(window, qtbot):
    """Test the Close All Tombs button on the Close tab."""
    window.pages.setCurrentIndex(2)
    button = window.close_page.close_all_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_open_page(window, qtbot, name, key, password):
    """Test tomb opening on the Open tab."""
    window.pages.setCurrentIndex(1)
    window.open_page.tomb_path.setText(name)
    window.open_page.key_path.setText(key)
    window.open_page.key_password.setText(password)
    window.open_page.debug = True
    button = window.open_page.open_button
    qtbot.mouseClick(button, Qt.LeftButton)
    assert window.open_page.message.text() == 'Tomb Opened Successfully'


def test_open_dialog(window, qtbot, monkeypatch):
    """Test the Set Tomb Path dialog on the Open page."""
    button = window.open_page.tomb_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/tomb", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.open_page.tomb_path.text() == "/path/to/test/tomb"


def test_resize_dialog(window, qtbot, monkeypatch):
    """Test the Set Tomb Path dialog on the Resize page."""
    button = window.resize_page.tomb_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/tomb", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.resize_page.tomb_path.text() == "/path/to/test/tomb"


def test_open_key_path(window, qtbot, monkeypatch):
    """Test the Select Key dialog on the Open page."""
    button = window.open_page.key_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/test.tomb.key", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.open_page.key_path.text() == "/path/to/test/test.tomb.key"


def test_resize_key_path(window, qtbot, monkeypatch):
    """Test the Select Key dialog on the Resize page."""
    button = window.resize_page.key_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/test.tomb.key", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.resize_page.key_path.text() == "/path/to/test/test.tomb.key"


def test_list_items(window, qtbot, monkeypatch):
    """Test the Update List Items on the List page."""
    button = window.list_page.update_list_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_advanced_key_path(window, qtbot, monkeypatch):
    """Test the Select Key dialog on the Advanced page."""
    button = window.advanced_page.key_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/test.tomb.key", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.advanced_page.key_path.text() == "/path/to/test/test.tomb.key"


def test_image_path(window, qtbot, monkeypatch):
    """Test the Select Image dialog on the Advanced page."""
    button = window.advanced_page.image_path_button
    
    def fake_get_open_file_name(*args, **kwargs):
        return ("/path/to/test/test.jpg", "All Files (*.*)")
    
    monkeypatch.setattr(QFileDialog, "getOpenFileName", fake_get_open_file_name)

    qtbot.mouseClick(button, Qt.LeftButton)

    assert window.advanced_page.image_path.text() == "/path/to/test/test.jpg"


def test_install_path(window, qtbot, monkeypatch, tmp_path):
    """Test the Tomb Install Path dialog on the Config page."""
    fake_file = tmp_path / 'tomb'
    fake_file.write_text("fake binary content")

    widget = window.config_page

    # Monkeypatch QFileDialog to return the directory containing the fake file
    monkeypatch.setattr(QFileDialog, "getExistingDirectory", lambda *args, **kwargs: str(tmp_path))

    # Monkeypatch os.path.isfile to simulate the expected file check
    monkeypatch.setattr(os.path, "isfile", lambda path: path == str(tmp_path))

    # Optionally, monkeypatch config writing if needed
    monkeypatch.setattr(widget, "user_config_file", tmp_path / "config.toml")

    # Spy on signal (optional)
    with qtbot.waitSignal(widget.tomb_path_changed, timeout=1000) as blocker:
        widget.select_tomb_install_path()

    assert widget.tomb_path_line.text() == str(tmp_path)
    assert blocker.args == [str(tmp_path)]


def test_close_page_force_close(window, qtbot):
    """Test the Force Close Tombs on the Close tab."""
    window.pages.setCurrentIndex(2)
    button = window.close_page.force_close_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_resize_page(window, qtbot, name, key, password):
    """Test tomb resizing on the Resize tab."""
    window.pages.setCurrentIndex(3)
    window.resize_page.tomb_path.setText(name)
    window.resize_page.key_path.setText(key)
    window.resize_page.key_password.setText(password)
    window.resize_page.size_box.setValue(50)
    window.resize_page.open_checkbox.setChecked(True)
    window.resize_page.debug = True
    button = window.resize_page.resize_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_close_page_force_close_resize(window, qtbot):
    """Test the Force Close Tombs on the Close tab."""
    window.pages.setCurrentIndex(2)
    button = window.close_page.force_close_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_sudo_checkbox(window):
    """Test that the sudo checkbox correctly adds/removes the sudo lineedit."""
    window.config_page.sudo_checkbox.setChecked(False)
    assert window.create_page.tomb_layout.rowCount() == 4
    assert window.open_page.open_layout.rowCount() == 3
    assert window.resize_page.resize_layout.rowCount() == 3

    window.config_page.sudo_checkbox.setChecked(True)
    assert window.create_page.tomb_layout.rowCount() == 5
    assert window.open_page.open_layout.rowCount() == 4
    assert window.resize_page.resize_layout.rowCount() == 4


def test_engrave_key(window, qtbot, key):
    """Test engraving the key inside a QR image."""
    window.pages.setCurrentIndex(5)
    window.advanced_page.key_path.setText(key)
    button = window.advanced_page.engrave_button
    qtbot.mouseClick(button, Qt.LeftButton)


def test_exhume_bury_key(window, qtbot, key, image_file, password):
    """Test burying the key inside the given image file."""
    window.pages.setCurrentIndex(5)
    window.advanced_page.key_path.setText(key)
    window.advanced_page.image_path.setText(image_file)
    window.advanced_page.key_password.setText(password)
    button = window.advanced_page.bury_button
    qtbot.mouseClick(button, Qt.LeftButton)

    window.pages.setCurrentIndex(5)
    window.advanced_page.image_path.setText(image_file)
    window.advanced_page.key_password.setText(password)
    button = window.advanced_page.exhume_button
    qtbot.mouseClick(button, Qt.LeftButton)



from click.testing import CliRunner
import pytest

from mausoleum import wrapper


@pytest.fixture
def name():
    """Use test.tomb as the tomb name to pass to test functions."""
    return 'test.tomb'


@pytest.fixture
def key():
    """Use test.tomb.key as the tomb key name to pass to test functions."""
    return 'test.tomb.key'


@pytest.fixture
def password():
    """Use SUPER_SECURE_PASSWORD as the tomb password to pass to test functions."""
    return 'SUPER_SECURE_PASSWORD'


def test_dig_tomb(name):
    """Test creation of a new tomb container with a size of 10mb."""
    wrapper.dig_tomb(name, 10)


def test_forge_tomb(key, password):
    """Test creation of a new tomb key for the created tomb container."""
    wrapper.forge_tomb(key, password, debug=True, kdf=1)


def test_lock_tomb(name, key, password):
    """Test locking the tomb container with the created key."""
    wrapper.lock_tomb(name, key, password, debug=True)


def test_construct_tomb(name, key, password,):
    """Test constructing a new tomb container."""
    wrapper.construct_tomb('test2.tomb', 20, 'test2.tomb.key', password, debug=True)


def test_cli_enter(name, password):
    """Test the enter CLI command."""
    runner = CliRunner()
    result = runner.invoke(wrapper.cli, ['enter', name, '--debug'], input=password)
    assert not result.exception


def test_list_tombs(password):
    """Test that opened tombs are discovered."""
    assert '[test]' in wrapper.list_tombs()[0]


def test_close_tomb():
    """Test closing the created tomb."""
    wrapper.close_tomb(name='test')
    assert wrapper.list_tombs() == []

def test_read_only(name, key, password):
    """Test opening a tomb in read-only mode."""
    wrapper.open_tomb(name, key, password, debug=True, read_only=True, mountpoint='/media/test/mountpoint')


def test_close_all_tombs(name, key, password):
    """Test closing a tomb by opening the created tomb container."""
    wrapper.close_tombs()


def test_tomb_slam(name, key, password):
    """Test force closing a tomb by opening the create tomb container."""
    wrapper.open_tomb(name, key, password, debug=True)
    wrapper.slam_tombs()


def test_resize_tomb(name, key, password):
    """Test resizing the created tomb to 20mb."""
    wrapper.resize_tomb(name, 20, key, password)


def test_resize_cli(name, password):
    """Test the resize CLI command."""
    runner = CliRunner()
    result = runner.invoke(wrapper.cli, ['alter', '--open', '--debug', name, '30'], input=password)
    assert not result.exception


def test_cli_construct_and_close(password):
    """Test the construct CLI command."""
    runner = CliRunner()
    result = runner.invoke(wrapper.cli, ['construct', 'test3.tomb', '10', '--open', '--debug'], input=password)
    assert not result.exception

    result = runner.invoke(wrapper.cli, ['list'])
    assert 'test3' in result.output

    result = runner.invoke(wrapper.cli, ['leave', 'test3'])
    assert not result.exception


def test_cli_escape(name, password):
    runner = CliRunner()
    result = runner.invoke(wrapper.cli, ['enter', name, '--debug'], input=password)

    assert not result.exception

    result = runner.invoke(wrapper.cli, ['escape'])

    assert not result.exception


def test_cli_mold(key):
    """Test the mold CLI command which uses the engrave function."""
    runner = CliRunner()
    result = runner.invoke(wrapper.cli, ['mold', key])
    assert not result.exception


def test_cli_etch_and_resurrect(image_file, key, password):
    """Test the etch CLI command which uses the bury function.

    This tests two things: the embedding of a key inside an image and the extraction
    of a key from that image
    """
    runner = CliRunner()

    result = runner.invoke(wrapper.cli, ['etch', image_file, key], input=password)
    assert not result.exception

    result = runner.invoke(wrapper.cli, ['resurrect', image_file], input=password)
    assert not result.exception



from setuptools import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.rst").read_text()


setup(
    name='mausoleum',
    use_scm_version={"write_to": "mausoleum/_version.py"},
    description='A Python GUI, CLI, and wrapper for Tomb',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Mandeep',
    license='GPLv3+',
    url='https://github.com/mandeep/Mausoleum',
    packages=['mausoleum', 'mausoleum.images'],
    package_data={'mausoleum.images': ['*.png'], 'mausoleum': ['*.toml']},
    entry_points={
        'console_scripts': [
            'mausoleum-gui=mausoleum.application:main',
            'mausoleum=mausoleum.wrapper:cli'
        ]
    },
    setup_requires=["setuptools_scm>=8.0"],
    install_requires=[
        'appdirs',
        'click',
        'pytoml',
        'pyqt5'
    ],
    extras_require={
        'tests': [
            'pytest',
            'pytest-cov',
            'pytest-qt',
            'pytest-xvfb',
        ]
    },
    keywords='Mausoleum',
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    ]
)
